@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2011-2015, Cédric Krier
+# Copyright (c) 2011-2018, Cédric Krier
 # Copyright (c) 2013-2014, Nicolas Évrard
-# Copyright (c) 2011-2015, B2CK
+# Copyright (c) 2011-2018, B2CK
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,14 +29,16 @@
 
 from __future__ import division
 
-__version__ = '0.8'
-__all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
-    'Asc', 'Desc', 'NullsFirst', 'NullsLast', 'format2numeric']
-
 import string
+import numbers
 import warnings
 from threading import local, currentThread
 from collections import defaultdict
+from itertools import chain
+
+__version__ = '1.2.0'
+__all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
+    'Asc', 'Desc', 'NullsFirst', 'NullsLast', 'format2numeric']
 
 
 def alias(i, letters=string.ascii_lowercase):
@@ -77,7 +79,7 @@ class Flavor(object):
 
     def __init__(self, limitstyle='limit', max_limit=None, paramstyle='format',
             ilike=False, no_as=False, no_boolean=False, null_ordering=True,
-            function_mapping=None):
+            function_mapping=None, filter_=False):
         assert limitstyle in ['fetch', 'limit', 'rownum']
         self.limitstyle = limitstyle
         self.max_limit = max_limit
@@ -87,6 +89,7 @@ class Flavor(object):
         self.no_boolean = no_boolean
         self.null_ordering = null_ordering
         self.function_mapping = function_mapping or {}
+        self.filter_ = filter_
 
     @property
     def param(self):
@@ -157,6 +160,14 @@ class AliasManager(object):
             return ''
         return cls.local.alias[id(from_)]
 
+    @classmethod
+    def contains(cls, from_):
+        if getattr(cls.local, 'alias', None) is None:
+            return False
+        if from_ in cls.local.exclude:
+            return False
+        return id(from_) in cls.local.alias
+
     @classmethod
     def set(cls, from_, alias):
         assert cls.local.alias.get(from_) is None
@@ -248,6 +259,10 @@ class FromItem(object):
     def alias(self):
         return AliasManager.get(self)
 
+    @property
+    def has_alias(self):
+        return AliasManager.contains(self)
+
     def __getattr__(self, name):
         if name.startswith('__'):
             raise AttributeError
@@ -293,7 +308,7 @@ class With(FromItem):
         super(With, self).__init__(**kwargs)
 
     def statement(self):
-        columns = ('(%s)' % ', '.join('"%s"' % c for c in self.columns)
+        columns = (' (%s)' % ', '.join('"%s"' % c for c in self.columns)
             if self.columns else '')
         return '"%s"%s AS (%s)' % (self.alias, columns, self.query)
 
@@ -346,7 +361,7 @@ class SelectQuery(WithQuery):
     @limit.setter
     def limit(self, value):
         if value is not None:
-            assert isinstance(value, (int, long))
+            assert isinstance(value, numbers.Integral)
         self._limit = value
 
     @property
@@ -356,7 +371,7 @@ class SelectQuery(WithQuery):
     @offset.setter
     def offset(self, value):
         if value is not None:
-            assert isinstance(value, (int, long))
+            assert isinstance(value, numbers.Integral)
         self._offset = value
 
     @property
@@ -382,26 +397,55 @@ class SelectQuery(WithQuery):
                 fetch = ' FETCH FIRST (%s) ROWS ONLY' % self.limit
             return offset + fetch
 
+    def as_(self, output_name):
+        return As(self, output_name)
+
 
 class Select(FromItem, SelectQuery):
     __slots__ = ('_columns', '_where', '_group_by', '_having', '_for_',
-        'from_')
+        'from_', '_distinct', '_distinct_on', '_windows')
 
     def __init__(self, columns, from_=None, where=None, group_by=None,
-            having=None, for_=None, **kwargs):
+            having=None, for_=None, distinct=False, distinct_on=None,
+            windows=None, **kwargs):
+        self._distinct = False
+        self._distinct_on = []
         self._columns = None
         self._where = None
         self._group_by = None
         self._having = None
         self._for_ = None
+        self._windows = []
         super(Select, self).__init__(**kwargs)
-        # TODO ALL|DISTINCT
+        self.distinct = distinct
+        self.distinct_on = distinct_on
         self.columns = columns
         self.from_ = from_
         self.where = where
         self.group_by = group_by
         self.having = having
         self.for_ = for_
+        self.windows = windows
+
+    @property
+    def distinct(self):
+        return bool(self._distinct or self._distinct_on)
+
+    @distinct.setter
+    def distinct(self, value):
+        self._distinct = bool(value)
+
+    @property
+    def distinct_on(self):
+        return self._distinct_on
+
+    @distinct_on.setter
+    def distinct_on(self, value):
+        if value is not None:
+            if isinstance(value, Expression):
+                value = [value]
+            assert all(isinstance(col, Expression) for col in value)
+        self._distinct_on = value
 
     @property
     def columns(self):
@@ -409,7 +453,7 @@ class Select(FromItem, SelectQuery):
 
     @columns.setter
     def columns(self, value):
-        assert all(isinstance(col, Expression) for col in value)
+        assert all(isinstance(col, (Expression, SelectQuery)) for col in value)
         self._columns = tuple(value)
 
     @property
@@ -458,20 +502,15 @@ class Select(FromItem, SelectQuery):
             assert all(isinstance(f, For) for f in value)
         self._for_ = value
 
-    @staticmethod
-    def _format_column(column):
-        if isinstance(column, As):
-            if Flavor.get().no_as:
-                return '%s %s' % (column.expression, column)
-            else:
-                return '%s AS %s' % (column.expression, column)
-        else:
-            return str(column)
-
-    def _window_functions(self):
+    @property
+    def windows(self):
         from sql.functions import WindowFunction
         from sql.aggregate import Aggregate
         windows = set()
+        if self._windows is not None:
+            for window in self._windows:
+                windows.add(window)
+                yield window
         for column in self.columns:
             window_function = None
             if isinstance(column, (WindowFunction, Aggregate)):
@@ -483,7 +522,30 @@ class Select(FromItem, SelectQuery):
             if (window_function and window_function.window
                     and window_function.window not in windows):
                 windows.add(window_function.window)
-                yield window_function
+                yield window_function.window
+
+    @windows.setter
+    def windows(self, value):
+        if value is not None:
+            assert all(isinstance(w, Window) for w in value)
+        self._windows = value
+
+    @staticmethod
+    def _format_column(column):
+        if isinstance(column, As):
+            if isinstance(column.expression, Select):
+                expression = '(%s)' % column.expression
+            else:
+                expression = column.expression
+            if Flavor.get().no_as:
+                return '%s %s' % (expression, column)
+            else:
+                return '%s AS %s' % (expression, column)
+        else:
+            if isinstance(column, Select):
+                return '(%s)' % column
+            else:
+                return str(column)
 
     def _rownum(self, func):
         aliases = [c.output_name if isinstance(c, As) else None
@@ -528,7 +590,24 @@ class Select(FromItem, SelectQuery):
             return self._rownum(str)
 
         with AliasManager():
-            from_ = str(self.from_)
+            if self.from_ is not None:
+                from_ = ' FROM %s' % self.from_
+            else:
+                from_ = ''
+
+            # format window before expressions to set alias
+            window = ', '.join(
+                '"%s" AS (%s)' % (w.alias, w) for w in self.windows)
+            if window:
+                window = ' WINDOW ' + window
+
+            if self.distinct:
+                distinct = 'DISTINCT '
+                if self.distinct_on:
+                    distinct += ('ON (%s) '
+                        % ', '.join(map(str, self.distinct_on)))
+            else:
+                distinct = ''
             if self.columns:
                 columns = ', '.join(map(self._format_column, self.columns))
             else:
@@ -542,16 +621,11 @@ class Select(FromItem, SelectQuery):
             having = ''
             if self.having:
                 having = ' HAVING ' + str(self.having)
-            window = ''
-            windows = [f.window for f in self._window_functions()]
-            if windows:
-                window = ' WINDOW ' + ', '.join(
-                    '"%s" AS (%s)' % (w.alias, w) for w in windows)
             for_ = ''
             if self.for_ is not None:
                 for_ = ' ' + ' '.join(map(str, self.for_))
             return (self._with_str()
-                + 'SELECT %s FROM %s' % (columns, from_)
+                + 'SELECT %s%s%s' % (distinct, columns, from_)
                 + where + group_by + having + window + self._order_by_str
                 + self._limit_offset_str + for_)
 
@@ -561,24 +635,30 @@ class Select(FromItem, SelectQuery):
                 and (self.limit is not None or self.offset is not None)):
             return self._rownum(lambda q: q.params)
         p = []
-        p.extend(self._with_params())
-        for column in self.columns:
-            if isinstance(column, As):
-                p.extend(column.expression.params)
-            p.extend(column.params)
-        p.extend(self.from_.params)
-        if self.where:
-            p.extend(self.where.params)
-        if self.group_by:
-            for expression in self.group_by:
-                p.extend(expression.params)
-        if self.order_by:
-            for expression in self.order_by:
-                p.extend(expression.params)
-        if self.having:
-            p.extend(self.having.params)
-        for window_function in self._window_functions():
-            p.extend(window_function.window.params)
+        with AliasManager():
+            # Set alias to window function to skip their params
+            for window_function in self.windows:
+                window_function.alias
+
+            p.extend(self._with_params())
+            for column in chain(self.distinct_on or (), self.columns):
+                if isinstance(column, As):
+                    p.extend(column.expression.params)
+                p.extend(column.params)
+            if self.from_ is not None:
+                p.extend(self.from_.params)
+            if self.where:
+                p.extend(self.where.params)
+            if self.group_by:
+                for expression in self.group_by:
+                    p.extend(expression.params)
+            if self.order_by:
+                for expression in self.order_by:
+                    p.extend(expression.params)
+            if self.having:
+                p.extend(self.having.params)
+            for window_function in self.windows:
+                p.extend(window_function.window.params)
         return tuple(p)
 
 
@@ -660,10 +740,13 @@ class Insert(WithQuery):
             # TODO manage DEFAULT
         elif self.values is None:
             values = ' DEFAULT VALUES'
-        returning = ''
-        if self.returning:
-            returning = ' RETURNING ' + ', '.join(map(str, self.returning))
         with AliasManager():
+            table = self.table
+            AliasManager.set(table, str(table)[1:-1])
+            returning = ''
+            if self.returning:
+                returning = ' RETURNING ' + ', '.join(
+                    map(self._format, self.returning))
             return (self._with_str()
                 + 'INSERT INTO %s' % self.table + columns
                 + values + returning)
@@ -733,7 +816,8 @@ class Update(Insert):
                 where = ' WHERE ' + str(self.where)
             returning = ''
             if self.returning:
-                returning = ' RETURNING ' + ', '.join(map(str, self.returning))
+                returning = ' RETURNING ' + ', '.join(
+                    map(self._format, self.returning))
             return (self._with_str()
                 + 'UPDATE %s SET ' % table + values + from_
                 + where + returning)
@@ -989,6 +1073,10 @@ class Join(FromItem):
     def alias(self):
         raise AttributeError
 
+    @property
+    def has_alias(self):
+        raise AttributeError
+
     def __getattr__(self, name):
         raise AttributeError
 
@@ -1185,6 +1273,9 @@ class Expression(object):
     def cast(self, typename):
         return Cast(self, typename)
 
+    def collate(self, collation):
+        return Collate(self, collation)
+
     @property
     def asc(self):
         return Asc(self)
@@ -1229,6 +1320,7 @@ class Literal(Expression):
                 return ()
         return (self._value,)
 
+
 Null = None
 
 
@@ -1240,6 +1332,8 @@ class _Rownum(Expression):
     @property
     def params(self):
         return ()
+
+
 _rownum = _Rownum()
 
 
@@ -1296,7 +1390,7 @@ class Cast(Expression):
     __slots__ = ('expression', 'typename')
 
     def __init__(self, expression, typename):
-        super(Expression, self).__init__()
+        super(Cast, self).__init__()
         self.expression = expression
         self.typename = typename
 
@@ -1315,6 +1409,31 @@ class Cast(Expression):
             return (self.expression,)
 
 
+class Collate(Expression):
+    __slots__ = ('expression', 'collation')
+
+    def __init__(self, expression, collation):
+        super(Collate, self).__init__()
+        self.expression = expression
+        self.collation = collation
+
+    def __str__(self):
+        if isinstance(self.expression, Expression):
+            value = self.expression
+        else:
+            value = Flavor.get().param
+        if '"' in self.collation:
+            raise ValueError("Wrong collation %s" % self.collation)
+        return '%s COLLATE "%s"' % (value, self.collation)
+
+    @property
+    def params(self):
+        if isinstance(self.expression, Expression):
+            return self.expression.params
+        else:
+            return (self.expression,)
+
+
 class Window(object):
     __slots__ = ('_partition', '_order_by', '_frame', '_start', '_end')
 
@@ -1370,7 +1489,7 @@ class Window(object):
     @start.setter
     def start(self, value):
         if value:
-            assert isinstance(value, (int, long))
+            assert isinstance(value, numbers.Integral)
         self._start = value
 
     @property
@@ -1380,13 +1499,17 @@ class Window(object):
     @end.setter
     def end(self, value):
         if value:
-            assert isinstance(value, (int, long))
+            assert isinstance(value, numbers.Integral)
         self._end = value
 
     @property
     def alias(self):
         return AliasManager.get(self)
 
+    @property
+    def has_alias(self):
+        return AliasManager.contains(self)
+
     def __str__(self):
         partition = ''
         if self.partition:
