@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2011-2015, Cédric Krier
+# Copyright (c) 2011-2016, Cédric Krier
 # Copyright (c) 2013-2014, Nicolas Évrard
-# Copyright (c) 2011-2015, B2CK
+# Copyright (c) 2011-2016, B2CK
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@ from copy import deepcopy
 
 from sql import Table, Join, Union, Literal, Flavor, For, With, Window, Select
 from sql.functions import Now, Function, Rank, DatePart
-from sql.aggregate import Min
+from sql.aggregate import Min, Max
 
 
 class TestSelect(unittest.TestCase):
@@ -59,6 +59,40 @@ class TestSelect(unittest.TestCase):
             'SELECT * FROM "t" AS "a" WHERE ("a"."c" = %s)')
         self.assertEqual(query.params, ('foo',))
 
+    def test_select_without_from(self):
+        query = Select([Literal(1)])
+        self.assertEqual(str(query), 'SELECT %s')
+        self.assertEqual(query.params, (1,))
+
+    def test_select_select(self):
+        query = Select([Select([Literal(1)])])
+        self.assertEqual(str(query), 'SELECT (SELECT %s)')
+        self.assertEqual(query.params, (1,))
+
+    def test_select_select_as(self):
+        query = Select([Select([Literal(1)]).as_('foo')])
+        self.assertEqual(str(query), 'SELECT (SELECT %s) AS "foo"')
+        self.assertEqual(query.params, (1,))
+
+    def test_select_distinct(self):
+        query = self.table.select(self.table.c, distinct=True)
+        self.assertEqual(
+            str(query), 'SELECT DISTINCT "a"."c" FROM "t" AS "a"')
+        self.assertEqual(query.params, ())
+
+    def test_select_distinct_on(self):
+        query = self.table.select(self.table.c, distinct_on=self.table.c)
+        self.assertEqual(
+            str(query), 'SELECT DISTINCT ON ("a"."c") "a"."c" FROM "t" AS "a"')
+        self.assertEqual(query.params, ())
+
+        query = self.table.select(
+            self.table.c, distinct_on=[self.table.a, self.table.b])
+        self.assertEqual(
+            str(query),
+            'SELECT DISTINCT ON ("a"."a", "a"."b") "a"."c" FROM "t" AS "a"')
+        self.assertEqual(query.params, ())
+
     def test_select_from_list(self):
         t2 = Table('t2')
         t3 = Table('t3')
@@ -386,6 +420,39 @@ class TestSelect(unittest.TestCase):
             'WINDOW "b" AS (PARTITION BY DATE_PART(%s, "a"."date_col"))')
         self.assertEqual(query.params, ('year',))
 
+        window = Window([self.table.c2])
+        query = self.table.select(
+            Max(self.table.c1, window=window)
+            / Min(self.table.c1, window=window))
+        self.assertEqual(str(query),
+            'SELECT (MAX("a"."c1") OVER (PARTITION BY "a"."c2") '
+            '/ MIN("a"."c1") OVER (PARTITION BY "a"."c2")) '
+            'FROM "t" AS "a"')
+        self.assertEqual(query.params, ())
+
+        window = Window([Literal(1)])
+        query = self.table.select(
+            Max(self.table.c1, window=window)
+            / Min(self.table.c1, window=window))
+        self.assertEqual(str(query),
+            'SELECT (MAX("a"."c1") OVER (PARTITION BY %s) '
+            '/ MIN("a"."c1") OVER (PARTITION BY %s)) '
+            'FROM "t" AS "a"')
+        self.assertEqual(query.params, (1, 1))
+
+        window1 = Window([self.table.c2])
+        window2 = Window([Literal(1)])
+        query = self.table.select(
+            Max(self.table.c1, window=window1)
+            / Min(self.table.c1, window=window2),
+            windows=[window1])
+        self.assertEqual(str(query),
+            'SELECT (MAX("a"."c1") OVER "b" '
+            '/ MIN("a"."c1") OVER (PARTITION BY %s)) '
+            'FROM "t" AS "a" '
+            'WINDOW "b" AS (PARTITION BY "a"."c2")')
+        self.assertEqual(query.params, (1,))
+
     def test_order_params(self):
         with_ = With(query=self.table.select(self.table.c,
                 where=(self.table.c > 1)))
